VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DirLister"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'=========
'DirLister Version 2
'=========
'
'A wrapper Class providing a global predeclared object for using
'VB's Dir() intrinsic function more easily.
'
'
'DirLister handles subdirectory descent without using recursion
'and provides a mechanism to retrieve files or files and folders
'with a one by one call.
'
'It can be easier to use than recursive methods and should be
'much lighter in terms of resources used.  While it uses a tree
'of Collection objects these are populated gently and trimmed
'back aggressively.
'
'
'PROPERTIES
'
'Busy As Boolean (read only)
'
'   Seldom used, but returns True when a search has been started
'   and not yet finished.  While True you cannot start another
'   search because it will interrupt the internal state of
'   VBA.Dir().
'
'
'METHODS
'
'Start(ByVal SearchPath As String, _
'      Optional ByVal Attributes As DLFileAttribute = dlNormal,
'      Optional ByVal FilePattern As String = "*.*", _
'      Optional ByVal Depth As Integer)
'
'   First call Start() passing the SearchPath, FilePattern, and
'   Attributes desired.
'
'   The pattern is applied only to file names and not subdirectory
'   names.
'
'   If Attributes has the dlDirectory bit set then directory names
'   are returned as well as file names.  If the dlRelativePaths
'   bit is set, then . and .. are returned as directory names and
'   returned.
'
'   Depth = 0 means "infinite" while Depth = 1 returns items
'   immediately below SearchPath.
'
'Dir(ByRef Path As String) As Boolean
'
'   After Start() you can call Dir() passing a String variable
'   to receive the file or directory name.  Dir() returns True
'   until there are no more names to be returned.
'
'Finish()
'
'   When Dir() returns False (or whenever you abandon  a search)
'   you must call Finish() to release the underlying handles and
'   reset for a possible new Start() call.
'
'
'NOTES
'
'   o The Class provides a GLOBAL PREDECLARED INSTANCE and you do
'     not need (or really want) to create additional instances of
'     it.  Since VBA.Dir() is stateful only one could be actively
'     in use at a time anyway.
'
'     This is accomplished by manually setting:
'
'           Attribute VB_PredeclaredId = True
'
'     ... in the .CLS modules source using a text editor.
'
'   o This Class is dependent upon the Class DirLNode which it
'     uses internally.
'
'   o The FilePattern pattern is much richer than using patterns
'     with Dir() calls.  You have the full range of VB6 "Like"
'     operator patterns.  The pattern operates on the simple name
'     part of retrieved file names and not the entire full path.
'
'   o Files and paths with Unicode symbols or blocked by security
'     will be silently skipped.
'
'   o Calling Dir() in your own code while a search has been
'     started will destroy the process because it changes the
'     internal state of Dir().  This is true for any use of Dir()
'     and not specific to this Class.
'

Public Enum DLFileAttribute
    dlNormal = vbNormal
    dlReadOnly = vbReadOnly
    dlHidden = vbHidden
    dlSystem = vbSystem
    dlVolume = vbVolume
    dlDirectory = vbDirectory
    dlArchive = vbArchive
    dlAlias = vbArchive
    dlRelativePaths = &H80000
End Enum
#If False Then
Dim dlNormal, dlReadOnly, dlHidden, dlSystem, dlVolume
Dim dlDirectory, dlArchive, dlAlias, dlRelativePaths
#End If

Private Current As DirLNode
Private mAttributes As DLFileAttribute
Private mBusy As Boolean
Private mDepth As Integer
Private mDepthInfinite As Boolean
Private mPattern As String

Public Property Get Busy() As Boolean
    Busy = mBusy
End Property

Public Sub Start(ByVal SearchPath As String, _
                 Optional ByVal Attributes As DLFileAttribute = dlNormal, _
                 Optional ByVal FilePattern As String = "*", _
                 Optional ByVal Depth As Integer)
    
    If mBusy Then Err.Raise 5, TypeName(Me), "Invalid call, already busy"
    If Depth < 0 Then Err.Raise 5, TypeName(Me), "Depth value must be 0 or greater"
    
    mBusy = True
    Set Current = New DirLNode
    If Right$(SearchPath, 1) = "\" Then
        SearchPath = Left$(SearchPath, Len(SearchPath) - 1)
    End If
    Current.Path = SearchPath
    mAttributes = Attributes
    mDepth = Depth
    mDepthInfinite = Depth = 0
    mPattern = FilePattern
End Sub

Public Function Dir(ByRef Path As String) As Boolean
    'Returns True if a path name was returned.  False when
    'the entire list of results has been returned.
    Dim DirReturn As String
    Dim PathAttr As VbFileAttribute
    Dim RelativePath As Boolean
    Dim SubNode As DirLNode
    
    If Not mBusy Then Err.Raise 5, TypeName(Me), "Invalid call, call Start() first"
    
    Do
        Select Case Current.State
            Case dlnNew
                DirReturn = VBA.Dir(Current.Path & "\", _
                                    (mAttributes And Not dlRelativePaths) Or vbDirectory)
            Case dlnPathValid
                DirReturn = VBA.Dir()
            'Implied action:
            'Case dlnPathEmpty
            '    DirReturn = ""
        End Select
        
        If Len(DirReturn) > 0 Then
            Path = Current.Path & "\" & DirReturn
            Current.State = dlnPathValid
            'Trap secured locations and Unicode names.
            On Error Resume Next
            PathAttr = GetAttr(Path)
            If Err.Number = 0 Then
                On Error GoTo 0
                If PathAttr And vbDirectory Then
                    'Include return of directory if requested:
                    Dir = mAttributes And dlDirectory
                    RelativePath = DirReturn = "." Or DirReturn = ".."
                    If RelativePath Then
                        Dir = Dir And CBool(mAttributes And dlRelativePaths)
                    End If
                    
                    If (mDepthInfinite Or (mDepth > 1)) And Not RelativePath Then
                        Set SubNode = New DirLNode
                        With SubNode
                            Set .Parent = Current
                            .Path = Path
                        End With
                        Current.HoldSubDir SubNode
                    End If
                Else
                    'Include return of matching file:
                    Dir = DirReturn Like mPattern
                End If
            End If
            On Error GoTo 0
        Else
            Current.State = dlnPathEmpty
            If Current.HasSubs Then
                Set Current = Current.RetrieveNextSubDir()
                mDepth = mDepth - 1
            Else
                Set Current = Current.Parent
                mDepth = mDepth + 1
            End If
        End If
    Loop Until Dir Or (Current Is Nothing)
End Function

Public Sub Finish()
    'Should be called whenever done with the current search.  Most
    'important if the search was not used to exhaustion, this frees
    'handles and state within VBA.Dir() by hitting the nul: device.
    If mBusy Then
        VBA.Dir "nul"
        Set Current = Nothing
        mBusy = False
    End If
End Sub

Private Sub Class_Terminate()
    Finish
End Sub
